/******************************************************************************
*
* project name:    TI-Chess
* file name:       generic.c
* initial date:    13/07/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@gmx.net      (design/graphics/beta testing)
* description:     generic drawing routines
*
* $Id: generic.c,v 1.9 2002/11/11 11:08:19 nussbaumer Exp $
*
******************************************************************************/

#include "hardware.h"    // MUST BE ALWAYS HERE ON THE FIRST LINE !!!!

#include <stdlib.h>
#include <string.h>
#include <graph.h>
#include <gray.h>

#include "generic.h"

SCR_RECT fullscreen = {{0, 0, 239, 127}}; // used for fullscreen clipping



/*===========================================================================*/
/* draw a line with given color from (x0,y0) to (x1,y1)                      */
/*===========================================================================*/
void DrawColorLine(short x0,short y0,short x1, short y1, short color) {
    SetPlane(LIGHT_PLANE);
    if (color == COLOR_WHITE || color == COLOR_DARKGRAY) {
        DrawLine(x0, y0, x1, y1, A_REVERSE);
    }
    else {
        DrawLine(x0, y0, x1, y1, A_NORMAL);
    }
    SetPlane(DARK_PLANE);
    if (color == COLOR_WHITE || color == COLOR_LIGHTGRAY) {
        DrawLine(x0, y0, x1, y1, A_REVERSE);
    }
    else {
        DrawLine(x0, y0, x1, y1, A_NORMAL);
    }
}



/*===========================================================================*/
/* draws a rectangle with given color                                        */
/* if fill == 1) the rectangle is filled                                     */
/*===========================================================================*/
void DrawColorRect(short x0,short y0,short x1, short y1, short color,short fill) {
    WIN_RECT wrect;
    SCR_RECT rect;
    void     (*df)(void*,void*,short);
    void*    parea;

    if (fill) {
        rect.xy.x0 = x0;
        rect.xy.y0 = y0;
        rect.xy.x1 = x1;
        rect.xy.y1 = y1;
        parea      = &rect;
        df         = (void(*)(void*,void*,short))ScrRectFill;
    }
    else {
        wrect.x0 = x0;
        wrect.y0 = y0;
        wrect.x1 = x1;
        wrect.y1 = y1;
        parea    = &wrect;
        df       = (void(*)(void*,void*,short))DrawClipRect;
    }

    SetPlane(LIGHT_PLANE);
    if (color == COLOR_WHITE || color == COLOR_DARKGRAY) {
        (*df)(parea,&fullscreen, A_REVERSE);
    }
    else {
        (*df)(parea,&fullscreen, A_NORMAL);
    }
    SetPlane(DARK_PLANE);
    if (color == COLOR_WHITE || color == COLOR_LIGHTGRAY) {
        (*df)(parea,&fullscreen, A_REVERSE);
    }
    else {
        (*df)(parea,&fullscreen, A_NORMAL);
    }
}



/*===========================================================================*/
/* invert a given area                                                       */
/*===========================================================================*/
void InvertGrayRect(short x0,short y0,short x1, short y1) {
    short p;
    SCR_RECT sr = {{x0,y0,x1,y1}};


    for (p=0;p<2;p++) {
        SetPlane(p);
        ScrRectFill(&sr,&fullscreen,A_XOR);
    }
}



/*===========================================================================*/
/* draws a popup window                                                      */
/*===========================================================================*/
void DrawPopup(short x0,short y0,short x1, short y1) {
    DrawColorRect(x0,y0,x1,y1,COLOR_BLACK,RECT_EMPTY);
    x0++,y0++,x1--,y1--;
    DrawColorRect(x0,y0,x1,y1,COLOR_DARKGRAY,RECT_EMPTY);
    x0++,y0++,x1--,y1--;
    DrawColorRect(x0,y0,x1,y1,COLOR_LIGHTGRAY,RECT_EMPTY);
    x0++,y0++,x1--,y1--;
    DrawColorRect(x0,y0,x1,y1,COLOR_WHITE,RECT_FILLED);
}



/*===========================================================================*/
/* utility function to draw a string to both planes                          */
/* NOTE: there exists to new styles which are handled as flags               */
/*       A_CENTERED ... the given font will be used and the string will be   */
/*                      drawn centered on the screen                         */
/*       A_SHADOWED ... if this flag is set in the attributes a lightgray    */
/*                      shadow will be drawn with +1 pixel offset in all     */
/*                      two directions                                       */
/*===========================================================================*/
void DrawString(short x,short y,const char* s,short font,short attr) {
    short do_shadow = attr & A_SHADOWED;

    FontSetSys(font);

    if (attr & A_CENTERED) {
        x = (STATIC_LCD_WIDTH-DrawStrWidth(s,font))/2;
        attr &= ~A_CENTERED;
    }

    attr &= ~A_SHADOWED;

    SetPlane(DARK_PLANE);
    DrawStr(x,y,s,attr);
    SetPlane(LIGHT_PLANE);
    DrawStr(x,y,s,attr);

    if (do_shadow) DrawStr(x+1,y+1,s,A_NORMAL);
}



extern unsigned char* __F_4x6_Font__;
extern unsigned char* __F_6x8_Font__;

//=============================================================================
// setup fast font 4x6 drawing functions
// Thanx to Lionel Debroux for the new implementation which takes up no
// additionally memory
//=============================================================================
extern short InitFastDraw(void);
asm (".xdef __F_4x6_Font__\n"\
".even\n"\
"__F_4x6_Font__:\n"\
".long 0\n"\
".xdef __F_6x8_Font__\n"\
".even\n"\
"__F_6x8_Font__:\n"\
".long 0\n"\
".xdef InitFastDraw\n"\
"InitFastDraw:\n"\
"    movem.l %d1-%d3/%a0-%a2,-(%sp)\n"\
"    movea.l 0xC8.w,%a2\n"\
"    cmpi.w #1000,-2(%a2)\n"\
"    bgt.s __Prepare_fonts_AMS_2xx__\n"\
"__Prepare_fonts_AMS_1xx__:\n"\
"    movea.l 0x1A9*4(%a2),%a0\n"\
"    lea 58(%a0),%a0\n"\
"    movea.w (%a0),%a1\n"\
"    adda.l %a0,%a1\n"\
"    lea __F_4x6_Font__,%a0\n"\
"    move.l 122(%a1),(%a0)+\n"\
"    move.l 106(%a1),(%a0)\n"\
"    bra.s __Prepare_fonts_end__\n"\
"__Prepare_fonts_AMS_2xx__:\n"\
"    movea.l 0x45D*4(%a2),%a0 | EV_runningApp\n"\
"    move.w (%a0),%d3\n"\
"    beq.s __Prepare_fonts_GetAttr__\n"\
"    movea.l 0x441*4(%a2),%a0  | HeapTable\n"\
"    lsl.w #2,%d3\n"\
"    movea.l 0(%a0,%d3.w),%a0\n"\
"    movea.l 20(%a0),%a0\n"\
"    bra.s _Prepare_fonts_GetAttr__\n"\
"__Prepare_fonts_GetAttr__:\n"\
"    lea 0xFF000000.l,%a0\n"\
"_Prepare_fonts_GetAttr__:\n"\
"    movea.l 0x3FA*4(%a2),%a2 | OO_CondGetAttr\n"\
"    pea __F_4x6_Font__\n"\
"    pea 0x300.w\n"\
"    pea (%a0)\n"\
"    jsr (%a2)\n"\
"    tst.w %d0\n"\
"    beq.s __Prepare_fonts_problem__\n"\
"    addq.l #4,8(%sp)\n"\
"    addq.w #1,6(%sp)\n"\
"    jsr (%a2)\n"\
"    tst.w %d0\n"\
"    beq.s __Prepare_fonts_problem__\n"\
"    lea 12(%sp),%sp\n"\
"__Prepare_fonts_end__:\n"\
"    moveq #1,%d0\n"\
"_Prepare_fonts_end__:\n"\
"    movem.l (%sp)+,%d1-%d3/%a0-%a2\n"\
"    rts\n"\
"__Prepare_fonts_problem__:\n"\
"    moveq #0,%d0\n"\
"    bra.s _Prepare_fonts_end__");


//=============================================================================
// returns width of given string
//=============================================================================
short FastStringWidth(const unsigned char* s) {
    short width = 0;
    while (*s) width += *(unsigned char*)&(__F_4x6_Font__[(short)(*s++)*6]);;
    return width;
}


//=============================================================================
// draws string using own sprites (about 4 times faster than DrawStrXY)
//=============================================================================
void FastDraw(short x,short y,const unsigned char* s) {
    unsigned char* sprite;
    long           addr1;
    long           addr2;
    unsigned short cnt;
    short          ytemp;

    while (*s) {
        sprite = &(__F_4x6_Font__[(short)(*s)*6])+1;
        ytemp  = (y<<5)-(y<<1)+((x>>3)&0x1e);

        addr1 = ((long)GetPlane(DARK_PLANE))+ytemp;
        addr2 = ((long)GetPlane(LIGHT_PLANE))+ytemp;
        cnt  = 24-(x&15);

        // unrolled loop for more speed ....
        // NOTE: We are using XOR here, so this function can also be used
        // for the "white on black" strings
        *(long*)addr1|=(long)(*sprite)<<cnt;
        *(long*)addr2|=(long)(*sprite++)<<cnt;
        addr1+=30;addr2+=30;
        *(long*)addr1|=(long)(*sprite)<<cnt;
        *(long*)addr2|=(long)(*sprite++)<<cnt;
        addr1+=30;addr2+=30;
        *(long*)addr1|=(long)(*sprite)<<cnt;
        *(long*)addr2|=(long)(*sprite++)<<cnt;
        addr1+=30;addr2+=30;
        *(long*)addr1|=(long)(*sprite)<<cnt;
        *(long*)addr2|=(long)(*sprite++)<<cnt;
        addr1+=30;addr2+=30;
        *(long*)addr1|=(long)(*sprite)<<cnt;
        *(long*)addr2|=(long)(*sprite)<<cnt;

        x+=*(sprite - 5);
        s++;
    }
}

//=============================================================================
// draws string using own sprites (about 4 times faster than DrawStrXY)
// CENTERED version
//=============================================================================
void FastDrawCentered(short y,const unsigned char* s) {
    FastDraw((C89_92(160,240)-FastStringWidth(s))>>1,y,s);
}


//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: generic.c,v $
// Revision 1.9  2002/11/11 11:08:19  nussbaumer
// minor improvement in character sprites retrievement
//
// Revision 1.8  2002/10/17 11:30:56  tnussb
// changes due to new font graphics data retrieving by Lionel Debroux
// (the sprite data is fetched from the AMS)
//
// Revision 1.7  2002/10/16 18:28:51  tnussb
// changes related to the complete new puzzle file support (see history.txt)
//
// Revision 1.6  2002/10/14 12:48:27  tnussb
// FastDraw() added
//
// Revision 1.5  2002/02/11 16:38:11  tnussb
// many changes due to "separate file compiling" restructuring
//
// Revision 1.4  2001/02/17 15:00:11  Thomas Nussbaumer
// changes due to new TIGCC version
//
// Revision 1.3  2000/12/19 13:55:29  Thomas Nussbaumer
// warnings stated by compiling with option -Wall fixed
//
// Revision 1.2  2000/08/12 15:31:12  Thomas Nussbaumer
// substitution keywords added
//
//
